/* depends on:
	jquery core 1.4.2 or later <http://jquery.com>
	jquery ui effects 1.8.2 or later <http://jqueryui.com> 
	jquery scrollstay plugin
	jquery placeheld plugin <http://plugins.jquery.com/project/placeheld>
*/


/* jQuery settings */

jQuery.ajaxSettings.traditional = true; /* for compatibility with Django */


/* jQuery preload */
(function($) {
  var cache = [];
  // Arguments are image paths relative to the current page.
  $.preLoadImages = function() {
    var args_len = arguments.length;
    for (var i = args_len; i--;) {
      var cacheImage = document.createElement('img');
      cacheImage.src = arguments[i];
      cache.push(cacheImage);
    }
  }
})(jQuery)


/* Array Remove - By John Resig (MIT Licensed) */
/* <http://ejohn.org/blog/javascript-array-remove/> */
Array.prototype.remove = function(from, to) {
	var rest = this.slice((to || from) + 1 || this.length);
	this.length = from < 0 ? this.length + from : from;
	return this.push.apply(this, rest);
};


Array.prototype.compare = function(to) {
	if (this.length != to.length) { return false; }
	var a = this.sort(),
		b = to.sort();
	for (var i = 0; i < a.length; i++) {
		if (a[i] !== b[i]) { 
			return false;
		}
	}
	return true;
};



var harvest = new function () {

/* Globals and constants */
var XHR_ROOT = '/opportunities/xhr/'; /* TODO: would be nice to get this from Django */
var RESULTS_URL = XHR_ROOT + 'results/'
var OPPORTUNITIES_URL = XHR_ROOT + 'opportunity/';

var MEDIA_PATH = '/media/' /* we should get this from Django somehow */
var MEDIA = { 'pkg'     :  { 'loading' : MEDIA_PATH+'img/pkg-status-loading.gif' },
              'results' :  { 'waiting' : MEDIA_PATH+'img/results-status-waiting.gif',
                             'loading' : MEDIA_PATH+'img/results-status-loading.gif' }
            }

var harvest_results;
var harvest_filters_list;

/* preload media that needs to appear quickly */
jQuery.preLoadImages( MEDIA['pkg']['loading'],
                      MEDIA['results']['waiting'],
                      MEDIA['results']['loading'] );


function Filter (dom_node) {
	/* Attaches to a .filter object in the document. Handles events
	   specific to each type of filter and provides some extra data
	   and utilities. */
	
	var filter = this; /* so we can access this from event handlers */
	
	$(dom_node).data('filter',this); /* so we can get here from the JQuery object */
	
	this.full_name = $(dom_node).attr('data-filter-fullname');
	this.parent_filter_node = $(dom_node).parent().closest('.filter');
	this.container = $(dom_node).closest('li'); /* the list item in the parent that this belongs to */
	this.value_node = $(dom_node).children('.filter-value');
	
	
	this.is_class = function (class_name) {
		return $(dom_node).hasClass(class_name);
	}
	
	
	this.get_value_serialized = function () {
		/* Returns the value of this filter, ready for the data input of 
		   jQuery's ajax function so it reaches Harvest's Django portion
		   in the format it expects.
		   Does nothing by default. Implement this in each
		   "class"-specific initializer below. */
		
		return null;
	}
	
	this.value_changed = function () {
		/* Should be called when the filter's value has changed.
		   This posts the new value to the global Results object */
		
		value = this.get_value_serialized();
		if ( value != null ) {
			harvest_results.post_query(this.full_name, value);
		}
	}
	
	
	/********/
	/* Ugly hacks for talking to the parent node */
	
	this.get_parent_filtergroup = function () {
		/* Returns the FilterGroup that is a parent of this filter.
		   Returns null if the parent doesn't exist or isn't a FilterGroup. */
		parent = null;
		if ( this.parent_filter_node ) {
			parent_test = this.parent_filter_node.data('filter');
			if ( parent_test && parent_test.is_class('filtergroup') ) parent = parent_test;
		} 
		return parent;
	}
	
	
	this.force_enable_prelight_add = function () {
		/* Tells the parent (if there is one) to create a prelight for
		   enabling this item.
		   We assume the parent is a FilterGroup, which is probably right. */
		parent_filter = this.get_parent_filtergroup();
		if ( parent_filter && ! parent_filter.item_is_selected(this.container) ) {
			/* only do this if we weren't already selected */
			parent_filter.toggle_item_prelight_add(this.container);
		}
	}
	
	this.force_enable_prelight_clear = function () {
		/* Tells the parent (if there is one) to remove the prelight for
		   enabling this item.
		   We assume the parent is a FilterGroup. */
		parent_filter = this.get_parent_filtergroup()
		
		if ( parent_filter ) parent_filter.toggle_item_prelight_clear(this.container);
	}
	
	
	this.force_enable = function () {
		/* Tells the parent (if there is one) to enable this item,
		   because it said so!
		   We assume the parent is a FilterGroup. */
		parent_filter = this.get_parent_filtergroup();
		if ( parent_filter && ! parent_filter.item_is_selected(this.container) ) {
			parent_filter.enable_item(this.container);
		}
	}
	
	
	/* Callback functions */
	this.enabled = function() {} /* called when this filter is enabled */
	this.disabled = function() {} /* called when this filter is disabled */
	
	
	/********/
	/* "Class"-specific initialization stuff. Setting up functions and
	   handlers in filter.value_node */
	
	if ( this.is_class('choicefilter') ) {
		var list_items = this.value_node.children('ul').children('li');
		
		
		/* Callback functions */
		this.item_enabled = function(item) {} /* called after enabling an item */
		this.item_disabled = function(item) {} /* called after disabling an item */
		
		
		this.get_value_serialized = function () {
			var item_ids = ['']; /* empty default so this overrides defaults in server when unset */
			/* we add the data-item-id attribute of all selected items */
			list_items.each(function () {
				if ($(this).attr('data-selected') != null) {
					item_ids.push($(this).attr('data-item-id'));
				}
			} );
			return item_ids;
		}
		
		
		this.item_is_selected = function (item) {
			return ( item.attr('data-selected') != null );
		}
		
		this.toggle_item_prelight_add = function (item) {
			if ( this.item_is_selected(item) ) {
				item.addClass('prelight-off');
			} else {
				item.addClass('prelight-on');
				item.children('.checkbox').html('&#10003;'); /* decoration :) */
				this.force_enable_prelight_add();
			}
		}
		
		this.toggle_item_prelight_clear = function (item) {
			/* remove the decoration in the checkbox if we added it */
			if ( ! this.item_is_selected(item) ) {
				item.children('.checkbox').html('');
			}
			
			item.removeClass('prelight-on');
			item.removeClass('prelight-off');
			this.force_enable_prelight_clear();
		}
		
		
		this.enable_item = function (item) {
			item.attr('data-selected','');
			item.children('.checkbox').html('&#10003;');
			
			this.force_enable();
			this.value_changed();
			
			this.item_enabled(item);
		}
		
		this.disable_item = function (item) {
			item.removeAttr('data-selected');
			item.children('.checkbox').html('');
			
			this.value_changed();
			
			this.item_disabled(item);
		}
		
		this.toggle_item = function (item) {
			if ( this.item_is_selected(item) ) {
				this.disable_item(item);
			} else {
				this.enable_item(item);
			}
		}
		
		
		this.get_item_toggle_node = function (item) {
			/* returns the node(s) used to toggle the given item in this filter */
			return item.children('a.item-toggle');
		}
		
		
		/* "Subclass" setup goes here because we need to do overrides before any movement happens */
		if ( this.is_class('filtergroup') ) {
			this.item_enabled = function (item) {
				/* Tell child it has been enabled. It might do something cool! */
				child_filter = item.children('.filter').data('filter');
				child_filter.enabled();
			}
			
			this.item_disabled = function (item) {
				/* Tell child it has been disabled. */
				child_filter = item.children('.filter').data('filter');
				child_filter.disabled();
			}
		
			this.get_item_toggle_node = function (item) {
				return item.children('.filter').children('.filter-label').children('a.item-toggle');
			}
		}
		
		
		/* setup the event handlers we described on all the items */
		list_items.each(function () {
			var item = $(this);
			toggle_node = filter.get_item_toggle_node(item);
			toggle_node.bind('mouseenter focusin', function () { filter.toggle_item_prelight_add(item); } );
			toggle_node.bind('mouseleave focusout', function () { filter.toggle_item_prelight_clear(item); } );
			
			toggle_node.bind('click', function () {
				filter.toggle_item(item);
				filter.toggle_item_prelight_clear(item);
				
				return false; /* stops link from being followed */
			} );
		});
	}
	
	
	if ( this.is_class('editfilter') ) {
		var edit_field = this.value_node.children('input');
		
		
		this.edit_field_changed = function() {
			this.force_enable();
			this.value_changed();
		}
		
		this.get_value_serialized = function () {
			return escape(edit_field.val());
		}
		
		this.enabled = function () {
			edit_field.focus();
		}
		
		/* we could handle 'change' instead of 'input' to only check value when typing has stopped */
		edit_field.bind('input', function () { filter.edit_field_changed(); } );
	}
	
	
	/* send initial state to Results object so it can behave neatly */
	value = this.get_value_serialized();
	if ( value != null ) {
		harvest_results.update_current_query(this.full_name, value);
	}
}


function Package (dom_node, opps_query, expanded_cb, collapsed_cb) {
	/* Created for each package inside the #results element */
	
	/* gtksourceview highlights "package" as an error, so we'll have to forego the convention */
	var pkg = this;
	
	this.id = $(dom_node).attr('data-package-id');
	this.details = $(dom_node).children('.sourcepackage-details');
	
	this.loading_xhr = null;
	this.is_expanded = false;
	
	
	this.show_status = function (status) {
		var indicator = dom_node.children('.sourcepackage-header').children('.status');
		if ( status ) {
			indicator.stop(true, true);
			indicator.html('<img src="'+MEDIA.pkg[status]+'" />');
			if ( this.visible_status == null ) {
				indicator.fadeIn(1500);
			}
		} else {
			/* stop the previous animation as it was; we don't expect it to finish usually */
			indicator.stop(true, false);
			indicator.fadeOut(150, function () {
				indicator.html('');
			});
		}
		
		this.visible_status = status;
	}
	
	this.hide_status = function (status) {
		if ( this.visible_status == status ) this.show_status(null);
	}
	
	
	this.expand = function () {
		if ( this.is_expanded ) return;
		
		var show = function () {
			dom_node.stop(true, true);
			pkg.details.stop(true, true);
			dom_node.switchClass('collapsed', 'expanded', 150);
			pkg.details.animate({height: 'show', opacity: 'show'}, 150);
		}
	
		/* run the callback saying this package will be expanded */
		this.is_expanded = true;
		if ( expanded_cb ) expanded_cb(this);
	
		/* first, get package details from the server if they aren't already here */
		/* this poorly assumes that this.details contains no text at all if we haven't hit the server */
		if ( $.trim(this.details.html()) == '' ) {
			this.show_status('loading');
			
			this.loading_xhr = $.ajax({
				type: "GET",
				url: RESULTS_URL + this.id, dataType: 'html',
				data: opps_query,
				complete: function (xhr, status) {
						pkg.hide_status('loading');
						pkg.loading_xhr = null;
					},
				success: function (data, status, xhr) {
						/* check that xhr.status is set. If it isn't, xhr.abort was called */
						if (xhr.status) {
							pkg.details.html(data);
							show();
						}
					},
				}); 
		} else {
			show();
		}
	}
	
	this.stop_loading = function () {
		/* stops loading package details */
		if ( this.loading_xhr != null ) this.loading_xhr.abort();
	}
	
	this.collapse = function () {
		this.stop_loading();
		if ( ! this.is_expanded ) return;
		
		/* run the callback saying this package will be collapsed */
		this.is_expanded = false;
		if ( collapsed_cb ) collapsed_cb(this);
		
		dom_node.stop(true, true);
		pkg.details.stop(true, true);	
		dom_node.switchClass('expanded', 'collapsed', 150);
		pkg.details.animate({height: 'hide', opacity: 'hide'}, 150);
	}
	
	
	
	/********/
	/* Object initialization */
	
	/* make sure the package knows it has been expanded */
	if ( dom_node.hasClass('expanded') ) {
		this.is_expanded = true;
		if ( expanded_cb ) expanded_cb(this);
	}
	
	dom_node.children('a.sourcepackage-header').bind('click', function () {
		if ( pkg.is_expanded || pkg.loading_xhr != null ) {
			pkg.collapse();
		} else {
			pkg.expand();
		}
	
		return false;
	});
}


function Results (dom_node) {
	/* Attached to the single #results-pane in the document. Receives
	   new query parameters from Filter objects, grabs and displays new
	   results in the contained #results element. */
	
	/* The current_query variable is set with the current querystring,
	   but some filters could already have been set with Harvest's
	   default parameters while the querystring doesn't mention them.
	   On its own, this isn't a problem, but it may lead to some odd
	   behaviour. We should fix it by setting current_query to include
	   all of the defaults in Harvest's Django portion. */
	
	var results = this;
	
	this.query_countdown = null;
	this.loading_xhr = null;
	
	this.current_query = {};
	this.future_query = {};
	
	this.container = $(dom_node);
	this.output = $(dom_node).children('#results');
	this.status_bubble = $(dom_node).children('#results-status');
	
	this.packages = {};
	this.expanded_pkgs = [];
	
	
	this.update_packages = function () {
		/* Perform bits of setup with a new list of sourcepackages */
		var results_packages = this.output.find('li.sourcepackage');
		
		var old_packages = this.packages;
		
		/* dereferences existing Packages. We expect that they'll be cleaned up by the GC */
		this.packages = {};
		this.expanded_pkgs = [];
		/* With this line, new results will appear with currently expanded
		   packages already expanded. Without it, more queries are run
		   later to do the same.
		   TODO: is the current approach better in terms of user experience and performance? */
		this.update_current_query('expand', this.expanded_pkgs);
		
		var pkg_expanded_cb = function (pkg) {
			results.expanded_pkgs.push(pkg.id);
		}
		
		var pkg_collapsed_cb = function (pkg) {
			var index = results.expanded_pkgs.indexOf(pkg.id);
			if ( index > -1 ) {
				results.expanded_pkgs.remove(index);
			}
		}
		
		results_packages.each(function () {
			var dom_node = $(this);
			var opps_query = results.current_query; /* would be nice to only send properties starting with opp: */
			
			var pkg = new Package(dom_node, opps_query,
			                      pkg_expanded_cb, pkg_collapsed_cb);
			results.packages[pkg.id] = pkg;
			
			var old_pkg = old_packages[pkg.id];
			if ( old_pkg ) {
				/* This package was listed before. Make sure new
				   listing has the same expanded / collapsed state */
				if ( old_pkg.is_expanded ) {
					pkg.expand();
				} else {
					pkg.collapse();
				}
			}
		});
	}
	
	
	this.show_status = function (status) {
		/* Shows the given status in the #status_bubble.
		   Set status to 'waiting', 'loading', or null to hide the box. */
		this.status_bubble.stop(true,true);
		if ( status ) {
			this.status_bubble.html('<div><img src="'+MEDIA.results[status]+'" /></div>');
			if ( this.visible_status == null ) {
				this.status_bubble.animate({height: 'show', opacity: 'show'}, 250);
				this.output.fadeTo(250, 0.75);
			}
		} else {
			this.status_bubble.animate({height: 'hide', opacity: 'hide'}, 150,
				function () {
					results.status_bubble.html('');
					results.output.fadeTo(250, 1);
				}
			);
		}
		
		this.visible_status = status;
	}
	
	this.hide_status = function (status) {
		/* Hides the status bubble if it shows the given status */
		if ( this.visible_status == status ) this.show_status(null);
	}
	
	
	this.load_results = function () {
		/* Begins loading results for future_query */
		var load_query = $.extend({}, this.current_query, this.future_query);
		
		results.show_status('loading');
		
		this.loading_xhr = $.ajax({
			type: "GET",
			url: RESULTS_URL, dataType: 'html',
			data: load_query,
			complete: function (xhr, status) {
					results.hide_status('loading');
					results.loading_xhr = null;
				},
			success: function (data, status, xhr) {
					/* We have to work around this jquery bug: http://dev.jquery.com/ticket/6173
					   success callback is still being called after xhr.abort() */
					if (xhr.status) {
						/* display the (plain html) results we have receieved */
						results.output.html(data);
						results.current_query = load_query;
						results.future_query = {};
						results.update_packages();
					
						/* display statistics in the footer */
						time_header = xhr.getResponseHeader('X-Django-Request-Time')
						if (time_header) {
							$('#requeststats').html('Results generated in '+parseFloat(time_header).toFixed(2) + ' seconds');
						}
					}
				},
			});
	}
	
	this.stop_loading = function () {
		/* stops loading, or returns silently if that is not happening */
		if ( this.loading_xhr != null ) this.loading_xhr.abort();
	}
	
	
	this.reset_countdown = function () {
		/* Sets the query countdown timer for 1.5s.
		   When the timer ends, we load the results for future_query. */
		this.stop_countdown(true);
		this.show_status('waiting');
		/* stop anything that is loading already */
		this.stop_loading();
		
		this.query_countdown = setTimeout(
			function (results) {
				results.load_results();
				results.query_countdown = null;
			}, 1500, this);
	}
	
	this.stop_countdown = function (wait_next) {
		/* Stops the query countdown timer. */
		clearTimeout(this.query_countdown);
		this.query_countdown = null;
		
		if (! wait_next) this.hide_status('waiting');
	}
	
	
	this.post_query = function (key, value) {
		/* Adds the given key/value pair to future_query. If it is
		   necessary, a timer is started to get the results for
		   future_query. */
		current_value = this.current_query[key];
		if ( this.current_query[key] == value ||
		     current_value instanceof Array && value instanceof Array && value.compare(current_value) ) {
			/* the user has reverted a change; value is the same as before */
			delete this.future_query[key];
			if ( $.isEmptyObject(this.future_query) ) {
				/* no point getting results, so stop and wait for new input. */
				this.stop_countdown();
				this.stop_loading();
			}
		} else {
			this.future_query[key] = value;
			/* start the timer from 0, waiting to apply the new future_query */
			this.reset_countdown();
		}
	}
	
	this.update_current_query = function (key, value) {
		/* Adds the given key/value pair as part of current_query. For
		   example, when we first load the page it is useful to make
		   sure we know the current state of all the filters. */
		this.current_query[key] = value;
	}
	
	
	/********/
	/* Object initialization */
	
	this.update_packages();
	this.status_bubble.show(); /* weird hack because jQuery's offset method doesn't support invisible elements */
	this.status_bubble.scrollStay({});
	this.status_bubble.hide();
}



function create_filters () {
	/* Creates all filters for the document at once. */
	$('#filters').scrollStay({ offset:12 });
	
	var filters_list = new Array();
	$('#filters .filter').each(function (i) {
		filters_list.push(new Filter(this));
	});
	
	return filters_list;
}

function create_results () {
	return new Results($('#results-pane'));
}



$(document).ready(function () {
	harvest_results = create_results();
	harvest_filters_list = create_filters();
	
	$('input[placeholder]').placeHeld();
});

}

