from harvest.filters import filters, containers
from harvest.common.ordereddict import OrderedDict
import models

class PkgNameFilter(filters.EditFilter):
    def process_queryset(self, queryset):
        return queryset.filter(name__startswith = self.get_value())

class PkgSetFilter(filters.ChoiceFilter):
    def default_choices_dict(self):
        choices_dict = OrderedDict()
        for s in models.PackageSet.objects.all(): #TODO: find a way to sort these
            choices_dict[s.name] = s
        return choices_dict
    
    def process_queryset(self, queryset):
        return queryset.filter(packagesets__in = self.get_selected_choices())



class OppFeaturedFilter(filters.Filter):
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__featured = True)

class OppListFilter(filters.ChoiceFilter):
    def default_choices_dict(self):
        choices_dict = OrderedDict()
        for l in models.OpportunityList.objects.all(): #TODO: find a way to sort these
            if l.active:
                choices_dict[l.name] = l
        return choices_dict
    
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__in = self.get_selected_choices())

    def render_html_value_choice(self, item_id):
        toggle_params = self.serialize(self.get_value_with_selection(item_id))
        item_href = self.get_system().get_url_with_parameters(toggle_params)
        title_attribute = ''
        help_html = ''
        if hasattr(self.choices_dict[item_id], 'description') and \
            self.choices_dict[item_id].description:
            title_attribute = 'title="%s"' % self.choices_dict[item_id].description
        if hasattr(self.choices_dict[item_id], 'explanation') and \
            self.choices_dict[item_id].explanation:
            help_html = '<a class="help" href="%s" target="_blank">?</a>' % \
                            self.choices_dict[item_id].explanation
        return '<a class="item-toggle" href="%s" %s>%s</a> %s' % (item_href, title_attribute, item_id, help_html)


#we don't really need to create a special type here, but it may be handy
class HarvestFilters(containers.FilterSystem):
    def __init__(self):
        super(HarvestFilters, self).__init__(
            [
                filters.FilterGroup('pkg', [
                    PkgNameFilter('name', name='Named'),
                    PkgSetFilter('set', name='Package sets')
                ], name='Packages' ),
                filters.FilterGroup('opp', [
                    OppFeaturedFilter('featured', name='Featured'),
                    OppListFilter('list', name='Selected')
                ], name='Opportunities' )
            ],
            default_parameters = { 'pkg' : 'set',
                                   'pkg.set' : ['ubuntu-desktop'],
                                   'opp' : 'list',
                                   'opp.list' : ['bitesize'] }
        )

