#!/usr/bin/python

from django.core.management.base import NoArgsCommand
from django.conf import settings
from django.db import transaction

from common import opportunity_lists
from opportunities.models import OpportunityList

import sys
import os

class Command(NoArgsCommand):
    help = "Pull opportunity lists and update them in the DB."

    def update_lists(self):
        data_dir = os.path.join(settings.PROJECT_PATH, "data")
        if not os.path.exists(data_dir):
            os.makedirs(data_dir)
        list_dir = opportunity_lists.pull_lists(data_dir)

        for (list_url, 
             list_description, 
             list_explanation) in opportunity_lists.read_lists(list_dir):
            op_list, created = OpportunityList.objects.get_or_create(url=list_url)
            op_list.description = list_description
            if list_explanation:
                op_list.explanation = list_explanation
            op_list.active = True
            op_list.name = self.chop_name(list_url)
            op_list.save()

    def chop_name(self, list_url):
        filename = os.path.basename(list_url)
        if filename.endswith(".csv"):
            return filename.split(".csv")[0]
        if filename.endswith(".csv.gz"):
            return filename.split(".csv.gz")[0]
        return filename

    def handle_noargs(self, **options):
        transaction.enter_transaction_management()
        try:
            if not settings.PROJECT_PATH:
                print >> sys.stderr, "PROJECT_PATH not set."
                sys.exit(1)
            transaction.managed(True)
            try:
                self.update_lists()
            except:
                transaction.rollback()
                raise
            else:
                transaction.commit()
            finally:
                transaction.managed(False)
        finally:
            transaction.leave_transaction_management()
