#!/usr/bin/python

from django.core.management.base import NoArgsCommand

from common import launchpad
from opportunities.models import SourcePackage, PackageSet

import sys

class Command(NoArgsCommand):
    help = "Pull packageset information from Launchpad."

    def handle_noargs(self, **options):
        lp = launchpad.lp_login()
        if not lp:
            sys.exit(1)
        lp_packagesets = launchpad.get_packagesets(lp)
        package_mapping = {}
        unseeded, created = PackageSet.objects.get_or_create(name="unseeded")
        if created:
            unseeded.save()
        for lp_packageset in lp_packagesets:
            packageset, created = PackageSet.objects.get_or_create(name=lp_packageset.name)
            if created:
                packageset.save()
            # record information which package is in which package sets
            for package_name in lp_packageset.getSourcesIncluded():
                if not package_mapping.has_key(package_name):
                    package_mapping[package_name] = set()
                package_mapping[package_name].add((lp_packageset.name))
        for package_name in package_mapping.keys():
            try:
                package = SourcePackage.objects.get(name=package_name)
            except SourcePackage.DoesNotExist:
                package = None
            if package:
                for packageset_name in package_mapping[package_name]:
                    try:
                        package.packagesets.get(name=packageset_name)
                    except PackageSet.DoesNotExist:
                        package.packagesets.add(PackageSet.objects.get(name=packageset_name))
                # if package is in mapping, it can't be in the unseeded
                # list any more
                if package.packagesets.filter(name="unseeded"):
                    package.packagesets.remove(unseeded)
                for packageset in package.packagesets.all():
                    if packageset.name not in package_mapping[package_name]:
                        package.packagesets.remove(PackageSet.objects.get(name=packageset))
                package.save()
        unseeded_packages = SourcePackage.objects.filter(packagesets__isnull=True)
        for u in unseeded_packages:
            u.packagesets.add(unseeded)
            u.save()
